@@ -13,17 +13,22 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+import base64
+
+import binascii
 import glob
 import logging
 import os
 
 from PyQt5.QtCore import (pyqtSignal, QObject, QProcess)
+from PyQt5.QtWidgets import QApplication
 
 from pre_workbench.bbuf_parsing import apply_grammar_on_bbuf
 from pre_workbench.configs import SettingsField, SettingsSection, registerOption, getValue
+from pre_workbench.guihelper import qApp
 from pre_workbench.objects import ByteBuffer, ByteBufferList, ReloadRequired
 from pre_workbench.structinfo.exceptions import invalid, incomplete
-from pre_workbench.structinfo.pcap_reader import read_pcap_file
+from pre_workbench.structinfo.pcap_reader import read_pcap_file, PcapFormats
 from pre_workbench.typeregistry import TypeRegistry
 from pre_workbench.tshark_helper import findTshark, PdmlToPacketListParser, findInterfaces
 
@@ -56,7 +61,7 @@ class FileDataSource(DataSource):
 	def getConfigFields():
 		return [
 			SettingsField("fileName", "File name", "text", {"fileselect": "open"}),
-			SettingsField("formatInfo", "Grammar definition", "text", {})
+			SettingsField("formatInfo", "Grammar definition", "text", {"listselectcallback":formatinfoSelect})
 		]
 
 	def startFetch(self):
@@ -82,7 +87,7 @@ class DirectoryOfBinFilesDataSource(DataSource):
 		return [
 			SettingsField("fileName", "Directory", "text", {"fileselect": "dir"}),
 			SettingsField("filePattern", "Search pattern", "text", {"default":"*"}),
-			SettingsField("formatInfo", "Grammar definition", "text", {})
+			SettingsField("formatInfo", "Grammar definition", "text", {"listselectcallback":formatinfoSelect})
 		]
 
 	def startFetch(self):
@@ -107,13 +112,79 @@ class DirectoryOfBinFilesDataSource(DataSource):
 		pass
 
 
+@DataSourceTypes.register(DisplayName = "CSV file")
+class CSVFileDataSource(DataSource):
+	@staticmethod
+	def getConfigFields():
+		return [
+			SettingsField("fileName", "File name", "text", {"fileselect":"open"}),
+			SettingsField("csvDelimiter", "CSV column delimiter", "text", {"default":","}),
+			SettingsField("csvQuoteChar", "CSV quote character", "text", {"default":"\""}),
+			SettingsField("csvHeaders", "CSV file contains header", "check", {"default":True}),
+			SettingsField("payloadColumnIdx", "Payload column index", "text", {"listselectcallback": CSVFileDataSource._headerSelect}),
+			SettingsField("payloadEncoding", "Payload encoding", "select", {"options":[("hex","hex"),("base64","base64"),]}),
+			SettingsField("formatInfo", "Grammar definition", "text", {"listselectcallback":formatinfoSelect}),
+		]
+
+	@staticmethod
+	def _headerSelect(dialog):
+		import csv
+		with open(dialog.values['fileName'], "r") as csvfile:
+			reader = csv.reader(csvfile, delimiter=dialog.values['csvDelimiter'], quotechar=dialog.values['csvQuoteChar'])
+			header = next(reader)
+			return ((str(i), f) for i, f in enumerate(header))
+
+	def _getPayloadDecoder(self):
+		if self.params['payloadEncoding'] == "hex":
+			return binascii.unhexlify
+		elif self.params['payloadEncoding'] == "base64":
+			return base64.b64decode
+
+	def _prepareMeta(self, row, headers):
+		if headers:
+			return dict(zip(headers, row))
+		else:
+			return {"col%d" % i: field for i, field in enumerate(row)}
+
+	def startFetch(self):
+		import csv
+		plist = ByteBufferList()
+		decoder = self._getPayloadDecoder()
+		payload_idx = int(self.params['payloadColumnIdx'])
+		with open(self.params['fileName'], "r") as csvfile:
+			reader = csv.reader(csvfile, delimiter=self.params['csvDelimiter'], quotechar=self.params['csvQuoteChar'])
+			try:
+				header = next(reader) if self.params['csvHeaders'] else None
+			except Exception as exc:
+				raise Exception("Failed to read header") from exc
+			for i, row in enumerate(reader):
+				try:
+					payload = row[payload_idx]
+					payload_decoded = decoder(payload)
+					plist.add(ByteBuffer(payload_decoded, metadata=self._prepareMeta(row, header)))
+				except IndexError as exc:
+					raise Exception("Failed to read row %d: can't decode payload at index %d because row only has %d columns" % (i, payload_idx, len(row))) from exc
+				except Exception as exc:
+					raise Exception("Failed to read row %d" % i) from exc
+
+		for bbuf in plist.buffers:
+			apply_grammar_on_bbuf(bbuf, self.params["formatInfo"])
+
+		self.on_finished.emit()
+		return plist
+
+	def cancelFetch(self):
+		# cancel reading file
+		pass
+
+
 @DataSourceTypes.register(DisplayName = "PCAP file")
 class PcapFileDataSource(DataSource):
 	@staticmethod
 	def getConfigFields():
 		return [
 			SettingsField("fileName", "File name", "text", {"fileselect":"open"}),
-			SettingsField("formatInfo", "Grammar definition", "text", {})
+			SettingsField("formatInfo", "Grammar definition", "text", {"listselectcallback":formatinfoSelect})
 		]
 
 	def startFetch(self):
@@ -259,3 +330,6 @@ class LivePcapCaptureDataSource(DataSource):
 		self.process.waitForFinished(500)
 		self.process.kill()
 
+def formatinfoSelect(dialog):
+	names = QApplication.instance().project.formatInfoContainer.definitions.keys()
+	return zip(names, names)
